package Benchmark::Perl::Formance::Cargo;
# git description: v0.06-4-ge3ba969

BEGIN {
  $Benchmark::Perl::Formance::Cargo::AUTHORITY = 'cpan:SCHWIGON';
}
{
  $Benchmark::Perl::Formance::Cargo::VERSION = '0.07';
}
# ABSTRACK: PerlFormance - Invariant cargo files

use warnings;
use strict;
use 5.006;


1; # End of Benchmark::Perl::Formance::Cargo

__END__
=pod

=encoding utf-8

=head1 NAME

Benchmark::Perl::Formance::Cargo

=head1 NAME

Benchmark::Perl::Formance::Cargo - PerlFormance - Invariant cargo files

=head1 ABOUT

This distribution carries large invariant data used for benchmarking
in Benchmark::Perl::Formance so that CPAN is not spammed on every new
version of Benchmark::Perl::Formance.

=head1 AUTHOR

Steffen Schwigon <ss5@renormalist.net>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2012 by Steffen Schwigon.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut

